package com.idtech.simpledemo;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Set;

import android.app.Activity;
import android.app.Dialog;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;

import com.idtechproducts.device.*;
import com.idtechproducts.device.ReaderInfo.CAPTURE_ENCODE_TYPE;
import com.idtechproducts.device.ReaderInfo.CAPTURE_ENCRYPT_TYPE;
import com.idtechproducts.device.ReaderInfo.EVENT_MSR_Types;

public class MainActivity extends Activity implements  OnReceiverListener{

	// declaring the instance of the MiniSmartII;
	private IDT_MiniSmartII myDevice = null;
	private TextView connectStatusTextView;  
	private TextView textLog;
	private TextView lcdLog;
	private Button btnGetFirmware;
	private Button btnStartEMV;
	private Button btnCancelEMV;
	private Handler handler = new Handler();
    private boolean isReaderConnected = false;
    private String info = "";
	private String detail = "";
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		handler = new Handler();		
		btnStartEMV = (Button)findViewById(R.id.btn_StartEMV);
		btnCancelEMV = (Button)findViewById(R.id.btn_CancelEMV);
		btnGetFirmware = (Button)findViewById(R.id.btn_getFirmware);
	    textLog = (TextView)findViewById(R.id.textLog);
	    lcdLog = (TextView)findViewById(R.id.lcdLog);
	    connectStatusTextView = (TextView)findViewById(R.id.status_text);
		if(myDevice!=null){
			myDevice.unregisterListen();
			myDevice.release();
			myDevice = null;
		}
		myDevice = new IDT_MiniSmartII(this,this);
		myDevice.registerListen();
		

		btnGetFirmware.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {
	        	info = "Getting Firmware\n";
	        	detail = "";
	        	handler.post(doUpdateStatus);
	        	StringBuilder sb = new StringBuilder();
	            int ret = myDevice.device_getFirmwareVersion(sb);
	            if (ret == ErrorCode.SUCCESS) {
					info += "Firmware Version: " + sb.toString();
					detail = "";
					handler.post(doUpdateStatus);
				}
				else {
					info += "GetFirmwareVersion: Failed\n";
					info += "Status: "+ myDevice.device_getResponseCodeString(ret)+"";
					detail = "";
					handler.post(doUpdateStatus);							
					}
	        }  
	    });
		
		btnStartEMV.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {;
	        detail = "";
	        info = "Starting EMV Transaction\n";
        	handler.post(doUpdateStatus);
	        	IDT_MiniSmartII.emv_allowFallback(true);
				myDevice.emv_startTransaction(1.00, 0.00, 0, 30, null, false);
	        }  
	    });
		
		btnCancelEMV.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {
	        	detail = "";
		        info = "Canceling EMV Transaction\n";
	        	handler.post(doUpdateStatus);
	        	ResDataStruct resData = new ResDataStruct();
				myDevice.emv_cancelTransaction(resData);
	        }  
	    });
		
		
	}

	@Override
	public void ICCNotifyInfo(byte[] arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void LoadXMLConfigFailureInfo(int arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigCompleted(StructConfigParameters arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigProgress(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	private Runnable doUpdateLabel = new Runnable()
	{
	    public void run()
	    {
	        if(!isReaderConnected){
	            connectStatusTextView.setText("MINISMARTII DISCONNECTED");
	        }
	        else{
	            connectStatusTextView.setText("MINISMARTII CONNECTED");
	        }
	    }
	};
	@Override
	public void deviceConnected() {
		isReaderConnected = true;
		handler.post(doUpdateLabel);		
	}

	@Override
	public void deviceDisconnected() {
		isReaderConnected = false;
	    handler.post(doUpdateLabel);		
	}
	
	
	@Override
	public void emvTransactionData(IDTEMVData emvData) {
		
		detail += Common.emvErrorCodes(emvData.result);
		detail += "\r\n";
		if (emvData.result == IDTEMVData.START_TRANS_SUCCESS)
			detail += "Start transaction response:\r\n";
		else if (emvData.result == IDTEMVData.GO_ONLINE)
			detail += "\r\nAuthentication response:\r\n";
		else 
			detail += "\r\nComplete Transaction response:\r\n";
		if (!emvData.unencryptedTags.isEmpty())
		{
			detail += "Unencrypted Tags:\r\n";
			Set<String> keys = emvData.unencryptedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.unencryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (!emvData.maskedTags.isEmpty())
		{
			detail += "Masked Tags:\r\n";
			Set<String> keys = emvData.maskedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.maskedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (!emvData.encryptedTags.isEmpty())
		{
			detail += "Encrypted Tags:\r\n";
			Set<String> keys = emvData.encryptedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.encryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		handler.post(doUpdateStatus);
		 if (emvData.result == IDTEMVData.GO_ONLINE){
			//Auto Complete
			 byte[] response = new byte[]{0x30,0x30};
			 myDevice.emv_completeTransaction(false, response, null, null,null);
		 }
		 else if (emvData.result == IDTEMVData.START_TRANS_SUCCESS){  
				//Auto Authenticate
			 myDevice.emv_authenticateTransaction(null);
			}
	
	}

	public void lcdDisplay(int mode, String[] lines, int timeout) {
		
				if (mode == 0x01) //Menu Display
				{
					//automatically select 1st application
					myDevice.emv_lcdControlResponse((byte)mode, (byte)0x01);
				}
				else if (mode == 0x08) //Language Menu Display
				{
					//automatically select first language
					myDevice.emv_lcdControlResponse((byte)mode, (byte)0x01);
				}
				else{
					ResDataStruct toData = new ResDataStruct();
					info = lines[0];
					handler.post(doUpdateStatus);
				}			
	}


	@Override
	public void msgAudioVolumeAjustFailed() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgRKICompleted(String arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgToConnectDevice() {
		// TODO Auto-generated method stub
		
	}

	private Runnable doUpdateStatus = new Runnable()
	{
		public void run()
		{		
			lcdLog.setText(info);
			textLog.setText(detail);
		}
	};
	@Override
	public void swipeMSRData(IDTMSRData card) {

		
	}

	@Override
	public void timeout(int arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void dataInOutMonitor(byte[] arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgBatteryLow() {
		// TODO Auto-generated method stub
		
	}

	
	
}
